!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   ta_setup_mod
!> @brief   setup variables and structures in TRJ_ANALYSIS
!! @authors Norio Takase (NT), Takaharu Mori (TM)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

module ta_setup_mod

  use ta_control_mod
  use ta_option_mod
  use ta_option_str_mod
  use trajectory_mod
  use output_mod
  use input_mod
  use trajectory_str_mod
  use output_str_mod
  use molecules_mod
  use molecules_str_mod
  use fileio_par_mod
  use fileio_psf_mod
  use fileio_pdb_mod
  use fileio_top_mod
  use fileio_gpr_mod
 
  implicit none
  private

  ! subroutines
  public  :: setup

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    setup
  !> @brief        setup variables and structures in TRJ_ANALYSIS
  !! @authors      NT, TM
  !! @param[in]    ctrl_data  : information of control parameters
  !! @param[inout] molecule   : molecule information
  !! @param[inout] trj_list   : trajectory file list information
  !! @param[inout] trajectory : trajectory information
  !! @param[inout] output     : output information
  !! @param[inout] option     : option information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine setup(ctrl_data, molecule, trj_list, trajectory, output, option)

    ! formal arguments
    type(s_ctrl_data),       intent(in)    :: ctrl_data
    type(s_molecule),        intent(inout) :: molecule
    type(s_trj_list),        intent(inout) :: trj_list
    type(s_trajectory),      intent(inout) :: trajectory
    type(s_output),          intent(inout) :: output
    type(s_option),          intent(inout) :: option

    ! local variables
    type(s_par)              :: par
    type(s_psf)              :: psf
    type(s_pdb)              :: ref
    type(s_top)              :: top
    type(s_gpr)              :: gpr


    ! input files
    !
    call input_files(ctrl_data%inp_info, psf, ref, gpr)


    ! define molecules
    !
    if (psf%num_atoms > 0) then
      call define_molecules(molecule, pdb=ref, psf=psf, top=top, par=par)
    else
      call define_molecules(molecule, pdb=ref)
    end if

    call dealloc_psf_all(psf)
    call dealloc_pdb_all(ref)


    ! setup trajectory
    !
    call setup_trajectory(ctrl_data%trj_info, molecule, trj_list, trajectory)


    ! setup output 
    !
    call setup_output(ctrl_data%out_info, output)


    ! setup option
    !
    call setup_option(ctrl_data%opt_info, molecule, gpr, option)


    call dealloc_gpr_all(gpr)

    return

  end subroutine setup

end module ta_setup_mod
