!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   at_enefunc_mod
!> @brief   define potential energy functions
!! @authors Yuji Sugita (YS), Takaharu Mori (TM), Chigusa Kobayashi (CK)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

module at_enefunc_mod

  use at_enefunc_pme_mod
  use at_enefunc_charmm_mod
  use at_enefunc_go_mod
  use at_energy_mod
  use at_restraints_str_mod
  use at_enefunc_str_mod
  use at_energy_str_mod
  use at_boundary_mod
  use molecules_str_mod
  use fileio_par_mod
  use fileio_gpr_mod
  use messages_mod
  use constants_mod

  implicit none
  private

  ! subroutines
  public  :: define_enefunc

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    define_enefunc
  !> @brief        a driver subroutine for defining potential energy functions
  !! @authors      TM, CK
  !! @param[in]    ene_info   : ENERGY section control parameters
  !! @param[in]    bound_info : BOUNDARY section control parameters
  !! @param[in]    par        : CHARMM PAR information
  !! @param[in]    gpr        : GO model parameter information
  !! @param[in]    molecule   : molecule information
  !! @param[in]    restraints : restraints information
  !! @param[out]   enefunc    : structure of enefunc
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine define_enefunc(ene_info, bound_info, par, gpr, molecule, &
                            restraints, enefunc)

    ! formal arguments
    type(s_ene_info),        intent(in)    :: ene_info 
    type(s_boundary_info),   intent(in)    :: bound_info
    type(s_par),             intent(in)    :: par
    type(s_gpr),             intent(in)    :: gpr
    type(s_molecule),        intent(in)    :: molecule
    type(s_restraints),      intent(in)    :: restraints
    type(s_enefunc),         intent(inout) :: enefunc


    call init_enefunc(enefunc)


    enefunc%forcefield   = ene_info%forcefield
    enefunc%output_style = ene_info%output_style

    enefunc%switchdist   = ene_info%switchdist
    enefunc%cutoffdist   = ene_info%cutoffdist
    enefunc%pairlistdist = ene_info%pairlistdist
    enefunc%dielec_const = ene_info%dielec_const
    enefunc%force_switch = ene_info%vdw_force_switch


    select case (enefunc%forcefield)

    case (ForcefieldCHARMM, ForcefieldCHARMM19, ForcefieldKBGO)

      call define_enefunc_charmm &
                         (ene_info, bound_info, par, molecule, &
                          restraints, enefunc)

    case (ForcefieldAAGO, ForcefieldCAGO)

      call define_enefunc_go &
                         (ene_info, gpr, molecule, restraints, enefunc)

    end select

    return

  end subroutine define_enefunc

end module at_enefunc_mod
