!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   at_enefunc_str_mod
!> @brief   structure of energy functions
!! @authors Yuji Sugita (YS), Takashi Imai(TI), Chigusa Kobayashi (CK), 
!!          Takao Yoda (TY)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

module at_enefunc_str_mod

  use messages_mod
  use constants_mod

  implicit none
  private

  ! structures

  type, public :: s_pme_enefunc
    real(wp)                      :: alpha      ! alpha
    real(wp)                      :: alpha2m    ! -alpha^2
    real(wp)                      :: alpha2sp   ! 2*alpha/sqrt(pi)
    real(wp)                      :: u_self     ! Ewald self energy
    real(wp)                      :: bs_fact    ! B-spline factor (1/(n-1)...2)
    real(wp)                      :: bs_fact3   ! bs_fact^3
    real(wp)                      :: bs_fact3d  ! bs_fact^3*(n-1)
    integer                       :: n_bs       ! Order of B-spline
    integer                       :: ngrid(3)   ! Number of grid
    integer                       :: istart_atom
    integer                       :: iend_atom
    real(wp),         allocatable :: b2(:,:)         ! b^2(hx,hy,hz)
    real(wp),         allocatable :: gx(:)
    real(wp),         allocatable :: gy(:)
    real(wp),         allocatable :: gz(:)
    real(wp),         allocatable :: vir_fact(:,:,:) ! -2*(1+G^2/4a)/G^2
    real(wp),         allocatable :: theta(:,:,:)    ! F^-1[Theta](hx,hy,hz)
    real(wp),         allocatable :: f(:,:)
    real(wp),         allocatable :: v(:,:)
    real(wp),         allocatable :: qdf(:,:,:)
    real(wp),         allocatable :: force_wk(:,:)
    real(wp),         allocatable :: qdf_recv(:,:,:,:)
    real(wp),         allocatable :: qdf_send(:,:,:,:)
    integer,          allocatable :: i_list(:)
    integer,          allocatable :: x_start1(:)
    integer,          allocatable :: x_end1(:)
    integer,          allocatable :: y_start(:)
    integer,          allocatable :: y_end(:)
    integer,          allocatable :: y_local(:)
    integer,          allocatable :: y_start1(:)
    integer,          allocatable :: y_end1(:)
    integer,          allocatable :: z_start(:)
    integer,          allocatable :: z_end(:)
    integer,          allocatable :: z_local(:)
    complex(wp),      allocatable :: ftqdf_localA(:,:,:)
    complex(wp),      allocatable :: ftqdf_localB(:,:,:)
  end type s_pme_enefunc

  type, public :: s_table_enefunc

    ! General table
    logical                       :: table
    integer                       :: table_order
    real(wp)                      :: density

    real(wp),         allocatable :: table_ene(:)
    real(wp),         allocatable :: table_grad(:)
    real(wp),         allocatable :: table_ecor(:)
    real(wp),         allocatable :: table_decor(:)
    real(wp),         allocatable :: dtable_ecor(:)
    real(wp),         allocatable :: dtable_decor(:)

    ! Water
    character(5)                  :: water_model
    integer                       :: num_water
    integer                       :: num_solute
    integer,          allocatable :: water_list(:,:)
    integer,          allocatable :: solute_list(:)
    integer,          allocatable :: solute_list_inv(:)

    real(wp),         allocatable :: table_ene_WW(:,:)
    real(wp),         allocatable :: table_de_WW(:,:)

    ! exclustion list
    !
    integer,          allocatable :: num_nonb_excl(:)
    integer,          allocatable :: nonb_excl_list(:)

  end type s_table_enefunc

  type, public :: s_enefunc

    integer                       :: forcefield
    integer                       :: output_style

    integer                       :: num_bonds
    integer                       :: num_angles
    integer                       :: num_ureys
    integer                       :: num_dihedrals
    integer                       :: num_impropers
    integer                       :: num_cmaps
    integer                       :: num_atom_cls
    integer                       :: num_contacts
    integer                       :: num_atoms_ref 
    integer                       :: num_restraintgroups
    integer                       :: num_restraintfuncs
    integer                       :: max_restraint_numatoms
    integer                       :: max_restraint_numgrps

    integer                       :: istart_bond,      iend_bond
    integer                       :: istart_angle,     iend_angle
    integer                       :: istart_urey,      iend_urey
    integer                       :: istart_dihedral,  iend_dihedral
    integer                       :: istart_improper,  iend_improper
    integer                       :: istart_cmap,      iend_cmap
    integer                       :: istart_contact,   iend_contact
    integer                       :: istart_restraint, iend_restraint

    ! bond (size = num_bonds)
    integer,          allocatable :: bond_list(:,:)
    real(wp),         allocatable :: bond_force_const(:)
    real(wp),         allocatable :: bond_dist_min(:)

    ! angle (size = num_angles)
    integer,          allocatable :: angl_list(:,:)
    real(wp),         allocatable :: angl_force_const(:)
    real(wp),         allocatable :: angl_theta_min(:)

    ! urey-bradley (size = num_ureys)
    integer,          allocatable :: urey_list(:,:)
    real(wp),         allocatable :: urey_force_const(:)
    real(wp),         allocatable :: urey_rmin(:)

    ! dihedral (size = num_dihedrals)
    integer,          allocatable :: dihe_list(:,:)
    real(wp),         allocatable :: dihe_force_const(:)
    integer,          allocatable :: dihe_periodicity(:)
    real(wp),         allocatable :: dihe_phase(:)

    ! improper (size = num_impropers)
    integer,          allocatable :: impr_list(:,:)
    real(wp),         allocatable :: impr_force_const(:)
    integer,          allocatable :: impr_periodicity(:)
    real(wp),         allocatable :: impr_phase(:)

    ! cmap (size = num_cmaps)
    !
    !   cmap_list(i,n) : atom list for cmap where (1 <= i <= 8 and
    !                        1 <= n <= psf%num_cross_terms)
    !
    !   cmap_resolution(n) : the number of cells in one direction
    !                        1 <= n <= psf%num_cross_terms)
    !
    !   cmap_coef(i,j,idih1,idih2,n) : the c_ij value for n-th cmap.
    !                    where (1 <= n <= psf%num_cross_terms)
    !                      ,(1 <= i, j <= 4) and 
    !                       (1 <= idih1, idih2 <= 24).
    integer,          allocatable :: cmap_list(:,:)
    integer,          allocatable :: cmap_resolution(:)
    integer,          allocatable :: cmap_type(:)
    real(wp),         allocatable :: cmap_coef(:,:,:,:,:)
    real(wp),         allocatable :: cmap_force(:,:,:)

    ! nonbonded 1-4 interactions (size = num_nb14)

    ! non-bonded (size = num_atom_cls)
    integer,          allocatable :: nonb_atom_cls(:)
    real(wp),         allocatable :: nb14_lj6(:,:)
    real(wp),         allocatable :: nb14_lj10(:,:)
    real(wp),         allocatable :: nb14_lj12(:,:)
    real(wp),         allocatable :: nonb_lj6(:,:)
    real(wp),         allocatable :: nonb_lj10(:,:)
    real(wp),         allocatable :: nonb_lj12(:,:)
    real(wp),         allocatable :: scee(:,:)
    real(wp),         allocatable :: scnb(:,:)

    ! non-bonded (size = num_atoms)
    integer,          allocatable :: num_nonb_excl(:)
    integer,          allocatable :: num_nb14_calc(:)
    integer,          allocatable :: nonb_excl_list(:)
    integer,          allocatable :: nb14_calc_list(:)
    real(wp),         allocatable :: work(:,:)

    ! native contact (size = num_contacts)
    integer,          allocatable :: contact_list(:,:)
    real(wp),         allocatable :: contact_lj_attractive(:)
    real(wp),         allocatable :: contact_lj_repulsive(:)

    ! non contact pairs
    real(wp)                      :: noncontact_lj_repulsive

    real(wp)                      :: switchdist
    real(wp)                      :: cutoffdist
    real(wp)                      :: pairlistdist
    real(wp)                      :: dielec_const

    real(wp)                      :: nb14_qq_scale

    ! flag for position restraint 
    logical                       :: restraint_posi
    ! flag for restraint calculation
    logical                       :: restraint_flag
    ! restraint group (size = num_restraintgroups)
    integer,          allocatable :: restraint_numatoms(:)
    integer,          allocatable :: restraint_atomlist(:,:)
    real(wp),         allocatable :: restraint_masscoef(:,:)

    ! restraint func (size = num_restraintfuncs)
    integer,          allocatable :: restraint_kind(:)
    integer,          allocatable :: restraint_grouplist(:,:)
    integer,          allocatable :: restraint_funcgrp(:)
    integer,          allocatable :: restraint_exponent_func(:)
    integer,          allocatable :: restraint_exponent_dist(:,:)
    real(wp),         allocatable :: restraint_weight_dist(:,:)
    real(wp),         allocatable :: restraint_const(:,:)
    real(wp),         allocatable :: restraint_ref(:,:)
    real(wp),         allocatable :: restraint_wcom1(:,:)
    real(wp),         allocatable :: restraint_wcom2(:,:)
    real(wp),         allocatable :: restraint_wcom3(:,:)
    real(wp),         allocatable :: restraint_wcom4(:,:)
    real(wp),         allocatable :: restraint_wcom5(:,:)
    real(wp),         allocatable :: restraint_wtmp(:,:)
    real(wp),         allocatable :: restraint_wdrt(:)
    real(wp),         allocatable :: restraint_wfrc(:,:)

    ! restraint func (size = num_atoms_ref)
    real(wp),         allocatable :: restraint_refcoord(:,:)

    type(s_table_enefunc)         :: table
    logical                       :: force_switch

    ! pme
    logical                       :: pme_use
    type(s_pme_enefunc)           :: pme

  end type s_enefunc

  ! parameter for allocatable variables
  integer,      public, parameter :: EneFuncBond        = 1
  integer,      public, parameter :: EneFuncAngl        = 2
  integer,      public, parameter :: EneFuncUrey        = 3
  integer,      public, parameter :: EneFuncDihe        = 4
  integer,      public, parameter :: EneFuncImpr        = 5
  integer,      public, parameter :: EneFuncCmap        = 6 
  integer,      public, parameter :: EneFuncNbon        = 7 
  integer,      public, parameter :: EneFuncCntc        = 8
  integer,      public, parameter :: EneFuncRefg        = 9
  integer,      public, parameter :: EneFuncReff        = 10
  integer,      public, parameter :: EneFuncRefw        = 11
  integer,      public, parameter :: EneFuncRefc        = 12
  integer,      public, parameter :: EneFuncTable       = 13
  integer,      public, parameter :: EneFuncTableWater  = 14
  integer,      public, parameter :: EneFuncTableSolute = 15
  integer,      public, parameter :: EneFuncCmapType    = 16
  integer,      public, parameter :: EneFuncPme         = 17

  ! parameters
  integer,      public, parameter :: ForcefieldCHARMM   = 1
  integer,      public, parameter :: ForcefieldKBGO     = 2
  integer,      public, parameter :: ForcefieldCHARMM19 = 3
  integer,      public, parameter :: ForcefieldAAGO     = 4
  integer,      public, parameter :: ForcefieldCAGO     = 5

  integer,      public, parameter :: OutputStyleGENESIS = 1
  integer,      public, parameter :: OutputStyleCHARMM  = 2
  integer,      public, parameter :: OutputStyleNAMD    = 3
  
  character(*), public, parameter :: ForceFieldTypes(3)  = (/'CHARMM  ', &
                                                             'KBGO    ', &
                                                             'CHARMM19'/)

  character(*), public, parameter :: OutputStyleTypes(3) = (/'GENESIS ', &
                                                             'CHARMM  ', &
                                                             'NAMD    '/)

  ! subroutines
  public  :: init_enefunc
  public  :: alloc_enefunc
  public  :: dealloc_enefunc
  public  :: dealloc_enefunc_all

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    init_enefunc
  !> @brief        initialize energy functions information
  !! @authors      YS, CK
  !! @param[out]   enefunc  : structure of potential energy function
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine init_enefunc(enefunc)

    ! formal arguments
    type(s_enefunc),         intent(inout) :: enefunc

    
    enefunc%num_bonds               = 0 
    enefunc%num_angles              = 0
    enefunc%num_ureys               = 0
    enefunc%num_dihedrals           = 0 
    enefunc%num_impropers           = 0 
    enefunc%num_cmaps               = 0
    enefunc%num_atom_cls            = 0
    enefunc%num_contacts            = 0
    enefunc%num_atoms_ref           = 0
    enefunc%num_restraintgroups     = 0 
    enefunc%num_restraintfuncs      = 0 
    enefunc%max_restraint_numatoms  = 0 
    enefunc%max_restraint_numgrps   = 0 

    enefunc%istart_bond             = 0
    enefunc%iend_bond               = 0
    enefunc%istart_angle            = 0
    enefunc%iend_angle              = 0
    enefunc%istart_urey             = 0
    enefunc%iend_urey               = 0
    enefunc%istart_dihedral         = 0
    enefunc%iend_dihedral           = 0
    enefunc%istart_improper         = 0
    enefunc%iend_improper           = 0
    enefunc%istart_cmap             = 0
    enefunc%iend_cmap               = 0
    enefunc%istart_contact          = 0
    enefunc%iend_contact            = 0
    enefunc%istart_restraint        = 0
    enefunc%iend_restraint          = 0

    enefunc%forcefield              = ForcefieldCHARMM
    enefunc%output_style            = OutputStyleCHARMM

    enefunc%noncontact_lj_repulsive = 0.0_wp

    enefunc%switchdist              = 0.0_wp
    enefunc%cutoffdist              = 0.0_wp
    enefunc%pairlistdist            = 0.0_wp
    enefunc%dielec_const            = 0.0_wp

    enefunc%pme_use                 = .false.

    enefunc%restraint_posi          = .false.
    enefunc%restraint_flag          = .false.
    
    enefunc%table%table             = .false.
    enefunc%table%density           = 0.0_wp
    enefunc%table%water_model       = ''
    enefunc%table%num_water         = 0
    enefunc%table%num_solute        = 0

    return

  end subroutine init_enefunc

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    alloc_enefunc
  !> @brief        allocate energy functions information
  !! @authors      YS, TY, CK
  !! @param[inout] enefunc   : structure of EneFunc information
  !! @param[in]    variable  : selected variable
  !! @param[in]    var_size  : size of the selected variable
  !! @param[in]    var_size2 : 2nd size of the selected variable (optional)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine alloc_enefunc(enefunc, variable, var_size, var_size2)

    ! formal arguments
    type(s_enefunc),         intent(inout) :: enefunc
    integer,                 intent(in)    :: variable
    integer,                 intent(in)    :: var_size
    integer,       optional, intent(in)    :: var_size2

    ! local variables
    integer                  :: alloc_stat
    integer                  :: dealloc_stat
    integer                  :: var_size3


    alloc_stat   = 0
    dealloc_stat = 0

    ! allocate selected variables
    !
    select case (variable)

    case(EneFuncBond)

      if (allocated(enefunc%bond_list)) then
        if (size(enefunc%bond_list(1,:)) == var_size) return
        deallocate(enefunc%bond_list,        &
                   enefunc%bond_force_const, &
                   enefunc%bond_dist_min,    &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%bond_list(2,var_size),      &
               enefunc%bond_force_const(var_size), &
               enefunc%bond_dist_min(var_size),    &
               stat = alloc_stat)

      enefunc%bond_list   (1:2,1:var_size) = 0
      enefunc%bond_force_const(1:var_size) = 0.0_wp
      enefunc%bond_dist_min   (1:var_size) = 0.0_wp

    case(EneFuncAngl)

      if (allocated(enefunc%angl_list)) then
        if (size(enefunc%angl_list(1,:)) == var_size) return
        deallocate(enefunc%angl_list,        &
                   enefunc%angl_force_const, &
                   enefunc%angl_theta_min,   &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%angl_list(3, var_size),     &
               enefunc%angl_force_const(var_size), &
               enefunc%angl_theta_min(var_size),   &
               stat = alloc_stat)

      enefunc%angl_list  (1:3, 1:var_size) = 0
      enefunc%angl_force_const(1:var_size) = 0.0_wp
      enefunc%angl_theta_min  (1:var_size) = 0.0_wp

    case(EneFuncUrey)

      if (allocated(enefunc%urey_list)) then
        if (size(enefunc%urey_list(1,:)) == var_size) return
        deallocate(enefunc%urey_list,        &
                   enefunc%urey_force_const, &
                   enefunc%urey_rmin,        &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%urey_list(2, var_size),     &
               enefunc%urey_force_const(var_size), &
               enefunc%urey_rmin(var_size),        &
               stat = alloc_stat)

      enefunc%urey_list  (1:2, 1:var_size) = 0
      enefunc%urey_force_const(1:var_size) = 0.0_wp
      enefunc%urey_rmin       (1:var_size) = 0.0_wp

    case(EneFuncDihe)

      if (allocated(enefunc%dihe_list)) then
        if (size(enefunc%dihe_list(1,:)) == var_size) return
        deallocate(enefunc%dihe_list,        &
                   enefunc%dihe_force_const, &
                   enefunc%dihe_periodicity, &
                   enefunc%dihe_phase,       &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%dihe_list(4,var_size),      &
               enefunc%dihe_force_const(var_size), &
               enefunc%dihe_periodicity(var_size), &
               enefunc%dihe_phase(var_size),       &
               stat = alloc_stat)

      enefunc%dihe_list   (1:4,1:var_size) = 0
      enefunc%dihe_force_const(1:var_size) = 0.0_wp
      enefunc%dihe_periodicity(1:var_size) = 0
      enefunc%dihe_phase      (1:var_size) = 0.0_wp

    case(EneFuncImpr)

      if (allocated(enefunc%impr_list)) then
        if (size(enefunc%impr_list(1,:)) == var_size) return
        deallocate(enefunc%impr_list,        &
                   enefunc%impr_force_const, &
                   enefunc%impr_periodicity, &
                   enefunc%impr_phase,       &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%impr_list(4,var_size),      &
               enefunc%impr_force_const(var_size), &
               enefunc%impr_periodicity(var_size), &
               enefunc%impr_phase(var_size),       &
               stat = alloc_stat)

      enefunc%impr_list   (1:4,1:var_size) = 0
      enefunc%impr_force_const(1:var_size) = 0.0_wp
      enefunc%impr_periodicity(1:var_size) = 0
      enefunc%impr_phase      (1:var_size) = 0.0_wp

    case(EneFuncCmap)

      if (allocated(enefunc%cmap_list)) then
        if (size(enefunc%cmap_list(1,:)) == var_size) return
        deallocate(enefunc%cmap_list,       &
                   enefunc%cmap_type,       &
                   enefunc%cmap_force,      &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%cmap_list(8,var_size),                       &
               enefunc%cmap_type(var_size),                         &
               enefunc%cmap_force(3,8,var_size),                  &
               stat = alloc_stat)

      enefunc%cmap_list(1:8,1:var_size)   = 0
      enefunc%cmap_type(1:var_size) = 0
      enefunc%cmap_force(1:3,1:8,1:var_size) = 0.0_wp

    case(EneFuncNbon)

      if (allocated(enefunc%nonb_atom_cls)) then
        if (size(enefunc%nonb_atom_cls(:)) == var_size) return
        deallocate(enefunc%nonb_atom_cls, &
                   enefunc%nb14_lj6,      &
                   enefunc%nb14_lj10,     &
                   enefunc%nb14_lj12,     &
                   enefunc%nonb_lj6,      &
                   enefunc%nonb_lj10,     &
                   enefunc%nonb_lj12,     &
                   enefunc%scee,          &
                   enefunc%scnb,          &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%nonb_atom_cls(var_size),      &
               enefunc%nb14_lj6(var_size,var_size),  &
               enefunc%nb14_lj10(var_size,var_size), &
               enefunc%nb14_lj12(var_size,var_size), &
               enefunc%nonb_lj6(var_size,var_size),  &
               enefunc%nonb_lj10(var_size,var_size), &
               enefunc%nonb_lj12(var_size,var_size), &
               enefunc%scee(var_size,var_size),      &
               enefunc%scnb(var_size,var_size),      &
               stat = alloc_stat)

      enefunc%nonb_atom_cls(1:var_size) = 0
      enefunc%nb14_lj6     (1:var_size,1:var_size) = 0.0_wp
      enefunc%nb14_lj10    (1:var_size,1:var_size) = 0.0_wp
      enefunc%nb14_lj12    (1:var_size,1:var_size) = 0.0_wp
      enefunc%nonb_lj6     (1:var_size,1:var_size) = 0.0_wp
      enefunc%nonb_lj10    (1:var_size,1:var_size) = 0.0_wp
      enefunc%nonb_lj12    (1:var_size,1:var_size) = 0.0_wp
      enefunc%scee         (1:var_size,1:var_size) = 0.0_wp
      enefunc%scnb         (1:var_size,1:var_size) = 0.0_wp

    case(EneFuncCntc)

      if (allocated(enefunc%contact_list)) then
        if (size(enefunc%contact_list(1,:)) == var_size) return
        deallocate(enefunc%contact_list,          &
                   enefunc%contact_lj_attractive, &
                   enefunc%contact_lj_repulsive,  &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%contact_list(2,var_size),        &
               enefunc%contact_lj_attractive(var_size), &
               enefunc%contact_lj_repulsive(var_size),  &
               stat = alloc_stat)

      enefunc%contact_list     (1:2,1:var_size) = 0
      enefunc%contact_lj_attractive(1:var_size) = 0.0_wp
      enefunc%contact_lj_repulsive (1:var_size) = 0.0_wp

    case(EneFuncRefg)

      if (allocated(enefunc%restraint_numatoms)) then
        if (size(enefunc%restraint_numatoms) == var_size) return
        deallocate(enefunc%restraint_numatoms, &
                   enefunc%restraint_atomlist, &
                   enefunc%restraint_masscoef, &
                   enefunc%restraint_wcom3,    &
                   enefunc%restraint_wcom4,    &
                   enefunc%restraint_wcom5,    &
                   enefunc%restraint_wtmp,     &
                   stat = dealloc_stat)
      end if
      allocate(enefunc%restraint_numatoms(var_size),               &
               enefunc%restraint_atomlist(1:var_size2,1:var_size), &
               enefunc%restraint_masscoef(1:var_size2,1:var_size), &
               enefunc%restraint_wcom3(1:3,1:var_size2),           &
               enefunc%restraint_wcom4(1:3,1:var_size2),           &
               enefunc%restraint_wcom5(1:3,1:var_size2),           &
               enefunc%restraint_wtmp(1:var_size2,1:var_size),     &
               stat = alloc_stat)

      enefunc%restraint_numatoms(1:var_size) = 0
      enefunc%restraint_atomlist(1:var_size2,1:var_size) = 0
      enefunc%restraint_masscoef(1:var_size2,1:var_size) = 0.0_wp
      enefunc%restraint_wcom3(1:3,1:var_size2) = 0.0_wp
      enefunc%restraint_wcom4(1:3,1:var_size2) = 0.0_wp
      enefunc%restraint_wcom5(1:3,1:var_size2) = 0.0_wp
      enefunc%restraint_wtmp(1:var_size2,1:var_size) = 0.0_wp

    case(EneFuncReff)

      if (allocated(enefunc%restraint_kind)) then
        if (size(enefunc%restraint_kind) == var_size) return
        deallocate(enefunc%restraint_kind,          &
                   enefunc%restraint_grouplist,     &
                   enefunc%restraint_const,         &
                   enefunc%restraint_ref,           &
                   enefunc%restraint_funcgrp,       &
                   enefunc%restraint_exponent_func, &
                   enefunc%restraint_exponent_dist, &
                   enefunc%restraint_weight_dist,   &
                   enefunc%restraint_wcom1,         &
                   enefunc%restraint_wcom2,         &
                   enefunc%restraint_wdrt,          &
                   stat = dealloc_stat)
      end if

      var_size3 = max(int(var_size2/2),1)
      allocate(enefunc%restraint_kind(var_size),                        &
               enefunc%restraint_grouplist(1:var_size2,1:var_size),     &
               enefunc%restraint_const(1:4,1:var_size),                 &
               enefunc%restraint_ref(1:2,1:var_size),                   &
               enefunc%restraint_funcgrp(1:var_size),                   &
               enefunc%restraint_exponent_func(1:var_size),             &
               enefunc%restraint_exponent_dist(1:var_size3,1:var_size), &
               enefunc%restraint_weight_dist(1:var_size3,1:var_size),   &
               enefunc%restraint_wcom1(1:3,1:var_size2),                &
               enefunc%restraint_wcom2(1:3,1:var_size2),                &
               enefunc%restraint_wdrt(1:var_size3),                     &
               stat = alloc_stat)

      enefunc%restraint_kind(1:var_size) = 0
      enefunc%restraint_grouplist(1:var_size2,1:var_size) = 0
      enefunc%restraint_const(1:4,1:var_size) = 0.0_wp
      enefunc%restraint_ref(1:2,1:var_size) = 0.0_wp
      enefunc%restraint_funcgrp(1:var_size) = 0
      enefunc%restraint_exponent_func(1:var_size) = 0
      enefunc%restraint_exponent_dist(1:var_size3,1:var_size) = 0
      enefunc%restraint_weight_dist(1:var_size3,1:var_size) = 0.0_wp
      enefunc%restraint_wcom1(1:3,1:var_size2) = 0.0_wp
      enefunc%restraint_wcom2(1:3,1:var_size2) = 0.0_wp
      enefunc%restraint_wdrt(1:var_size3) = 0.0_wp

    case(EneFuncRefw)

      if (allocated(enefunc%restraint_wfrc)) then
        if (size(enefunc%restraint_wfrc(1,:)) == var_size) return
        deallocate(enefunc%restraint_wfrc, &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%restraint_wfrc(1:3,var_size), &
               stat = alloc_stat)

      enefunc%restraint_wfrc(1:3,1:var_size) = 0.0_wp

    case(EneFuncRefc)

      if (allocated(enefunc%restraint_refcoord)) then
        if (size(enefunc%restraint_refcoord) == var_size) return
        deallocate(enefunc%restraint_refcoord, &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%restraint_refcoord(1:3,var_size), &
               stat = alloc_stat)

      enefunc%restraint_refcoord(1:3,1:var_size) = 0.0_wp

    case(EneFuncTable)

      if (allocated(enefunc%table%table_ene_WW)) then
        if (size(enefunc%table%table_ene(:)) /= 6*var_size) &
          deallocate(enefunc%table%table_ene,    &
                     enefunc%table%table_grad,   &
                     enefunc%table%table_ecor,   &
                     enefunc%table%table_decor,  &
                     enefunc%table%dtable_ecor,  &
                     enefunc%table%dtable_decor, &
                     enefunc%table%table_ene_WW, &
                     enefunc%table%table_de_WW,  &
                     stat = dealloc_stat)
      end if

      if (.not. allocated(enefunc%table%table_ene_WW)) &
        allocate(enefunc%table%table_ene   (6*var_size),    &
                 enefunc%table%table_grad  (6*var_size),    &
                 enefunc%table%table_ecor  (var_size),      &
                 enefunc%table%table_decor (var_size),      &
                 enefunc%table%dtable_ecor (var_size),      &
                 enefunc%table%dtable_decor(var_size),      &
                 enefunc%table%table_ene_WW(6*var_size, 3), &
                 enefunc%table%table_de_WW (2*var_size, 3), &
                 stat = alloc_stat)

      enefunc%table%table_ene   (1:6*var_size)      = 0.0_wp
      enefunc%table%table_grad  (1:6*var_size)      = 0.0_wp
      enefunc%table%table_ecor  (1:var_size)        = 0.0_wp
      enefunc%table%table_decor (1:var_size)        = 0.0_wp
      enefunc%table%dtable_ecor (1:var_size)        = 0.0_wp
      enefunc%table%dtable_decor(1:var_size)        = 0.0_wp
      enefunc%table%table_ene_WW(1:6*var_size, 1:3) = 0.0_wp
      enefunc%table%table_de_WW (1:2*var_size, 1:3) = 0.0_wp

    case(EneFuncTableWater)

      if (allocated(enefunc%table%water_list)) then
        if (size(enefunc%table%water_list(1,:)) == var_size) return
        deallocate(enefunc%table%water_list, &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%table%water_list(3,var_size), &
               stat = alloc_stat)

      enefunc%table%water_list(1:3,1:var_size) = 0

    case(EneFuncTableSolute)

      if (allocated(enefunc%table%solute_list)) then
        if (size(enefunc%table%solute_list(:)) == var_size) return
        deallocate(enefunc%table%solute_list,     &
                   enefunc%table%solute_list_inv, &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%table%solute_list(var_size),      &
               enefunc%table%solute_list_inv(var_size2), &
               stat = alloc_stat)

      enefunc%table%solute_list    (1:var_size)  = 0
      enefunc%table%solute_list_inv(1:var_size2) = 0

    case(EneFuncCmapType)

      if (allocated(enefunc%cmap_resolution)) then
        if (size(enefunc%cmap_resolution(:)) == var_size) return
        deallocate(enefunc%cmap_resolution, &
                   enefunc%cmap_coef,       &
                   stat = dealloc_stat)
      end if

      allocate(enefunc%cmap_resolution(var_size),                   &
               enefunc%cmap_coef(4,4,var_size2,var_size2,var_size), &
               stat = alloc_stat)

      enefunc%cmap_resolution(1:var_size) = 0
      enefunc%cmap_coef(1:4,1:4,1:var_size2,1:var_size2,1:var_size) = 0.0_wp

    case default

      call error_msg('Alloc_Enefunc> bad variable')

    end select

    if (alloc_stat /= 0)   call error_msg_alloc
    if (dealloc_stat /= 0) call error_msg_dealloc    

    return

  end subroutine alloc_enefunc

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    dealloc_enefunc
  !> @brief        deallocate energy functions information
  !! @authors      YS, TY, CK
  !! @param[inout] enefunc  : structure of EneFunc information
  !! @param[in]    variable : selected variable
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine dealloc_enefunc(enefunc, variable)

    ! formal arguments
    type(s_enefunc),         intent(inout) :: enefunc
    integer,                 intent(in)    :: variable

    ! local variables
    integer                  :: dealloc_stat

  
    dealloc_stat = 0

    select case (variable)

    case(EneFuncBond)
      
      if (allocated(enefunc%bond_list)) then
        deallocate(enefunc%bond_list,        &
                   enefunc%bond_force_const, &
                   enefunc%bond_dist_min,    &
                   stat = dealloc_stat)
      end if

    case(EneFuncAngl)

      if (allocated(enefunc%angl_list)) then
        deallocate(enefunc%angl_list,           &
                   enefunc%angl_force_const,    &
                   enefunc%angl_theta_min,      &
                   stat = dealloc_stat)
      end if

    case(EneFuncUrey)

      if (allocated(enefunc%urey_list)) then
        deallocate(enefunc%urey_list,        &
                   enefunc%urey_force_const, &
                   enefunc%urey_rmin,        &
                   stat = dealloc_stat)
      end if

    case(EneFuncDihe)

      if (allocated(enefunc%dihe_list)) then
        deallocate(enefunc%dihe_list,        &
                   enefunc%dihe_force_const, &
                   enefunc%dihe_periodicity, &
                   enefunc%dihe_phase,       &
                   stat = dealloc_stat)
      end if

    case(EneFuncImpr)

      if (allocated(enefunc%impr_list)) then
        deallocate(enefunc%impr_list,        &
                   enefunc%impr_force_const, &
                   enefunc%impr_periodicity, &
                   enefunc%impr_phase,       &
                   stat = dealloc_stat)
      end if

    case(EneFuncCmap)

      if (allocated(enefunc%cmap_list)) then
        deallocate(enefunc%cmap_list,        &
                   enefunc%cmap_resolution,  &
                   enefunc%cmap_coef,        &
                   enefunc%cmap_force,       &
                   stat = dealloc_stat )
      end if

    case(EneFuncNbon)

      if (allocated(enefunc%nonb_atom_cls)) then
        deallocate(enefunc%nonb_atom_cls, &
                   enefunc%nb14_lj6,      &
                   enefunc%nb14_lj10,     &
                   enefunc%nb14_lj12,     &
                   enefunc%nonb_lj6,      &
                   enefunc%nonb_lj10,     &
                   enefunc%nonb_lj12,     &
                   enefunc%scee,          &
                   enefunc%scnb,          &
                   stat = dealloc_stat)
      end if

    case(EneFuncCntc)

      if (allocated(enefunc%contact_list)) then
        deallocate(enefunc%contact_list,          &
                   enefunc%contact_lj_attractive, &
                   enefunc%contact_lj_repulsive,  &
                   stat = dealloc_stat)
      end if

    case(EneFuncRefg)

      if (allocated(enefunc%restraint_numatoms)) then
        deallocate(enefunc%restraint_numatoms, &
                   enefunc%restraint_atomlist, &
                   enefunc%restraint_masscoef, &
                   enefunc%restraint_wcom3,    &
                   enefunc%restraint_wcom4,    &
                   enefunc%restraint_wcom5,    &
                   enefunc%restraint_wtmp,     &
                   stat = dealloc_stat)
      end if

    case(EneFuncReff)

      if (allocated(enefunc%restraint_kind)) then
        deallocate(enefunc%restraint_kind,           &
                   enefunc%restraint_grouplist,      &
                   enefunc%restraint_const,          &
                   enefunc%restraint_ref,            &
                   enefunc%restraint_funcgrp,        &
                   enefunc%restraint_exponent_func,  &
                   enefunc%restraint_exponent_dist,  &
                   enefunc%restraint_weight_dist,    &
                   enefunc%restraint_wcom1,          &
                   enefunc%restraint_wcom2,          &
                   enefunc%restraint_wdrt,           &
                   stat = dealloc_stat)
      end if

    case(EneFuncRefw)

      if (allocated(enefunc%restraint_wfrc)) then
        deallocate(enefunc%restraint_wfrc, &
                   stat = dealloc_stat)
      end if

    case(EneFuncRefc)

      if (allocated(enefunc%restraint_refcoord)) then
        deallocate(enefunc%restraint_refcoord, &
                   stat = dealloc_stat)
      end if

    case(EneFuncTable)

      if (allocated(enefunc%table%table_ene_WW)) then
        deallocate(enefunc%table%table_ene,    &
                   enefunc%table%table_grad,   &
                   enefunc%table%table_ecor,   &
                   enefunc%table%table_decor,  &
                   enefunc%table%dtable_ecor,  &
                   enefunc%table%dtable_decor, &
                   enefunc%table%table_ene_WW, &
                   enefunc%table%table_de_WW,  &
                   stat = dealloc_stat)
      end if

    case(EneFuncTableWater)

      if (allocated(enefunc%table%water_list)) then
        deallocate(enefunc%table%water_list, &
                   stat = dealloc_stat)
      end if

    case(EneFuncTableSolute)

      if (allocated(enefunc%table%solute_list)) then
        deallocate(enefunc%table%solute_list,     &
                   enefunc%table%solute_list_inv, &
                   stat = dealloc_stat)
      end if

    case(EneFuncCmapType)

      if (allocated(enefunc%cmap_resolution)) then
        deallocate(enefunc%cmap_resolution, &
                   enefunc%cmap_coef,       &
                   stat = dealloc_stat)
      end if

    case(EneFuncPme) 

      if (allocated(enefunc%pme%gx)) then
        deallocate(enefunc%pme%gx,           &
                   enefunc%pme%gy,           &
                   enefunc%pme%gz,           &
                   enefunc%pme%b2,           &
                   enefunc%pme%vir_fact,     &
                   enefunc%pme%theta,        &
                   enefunc%pme%ftqdf_localA, &
                   enefunc%pme%ftqdf_localB, &
                   enefunc%pme%qdf,          &
                   enefunc%pme%qdf_recv,     &
                   enefunc%pme%qdf_send,     &
                   enefunc%pme%f,            &
                   enefunc%pme%v,            &
                   enefunc%pme%i_list,       &
                   enefunc%pme%force_wk,     &
                   enefunc%pme%x_start1,     &
                   enefunc%pme%x_end1,       &
                   enefunc%pme%y_start,      &
                   enefunc%pme%y_end,        &
                   enefunc%pme%y_local,      &
                   enefunc%pme%y_start1,     &
                   enefunc%pme%y_end1,       &
                   enefunc%pme%z_start,      &
                   enefunc%pme%z_end,        &
                   enefunc%pme%z_local,      &
                   stat = dealloc_stat)
      end if

    case default

      call error_msg('Dealloc_Enefunc> bad variable')

    end select

    if (dealloc_stat /= 0) call error_msg_dealloc

    return

  end subroutine dealloc_enefunc

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    dealloc_enefunc_all
  !> @brief        deallocate all energy functions information
  !! @authors      YS, CK
  !! @param[inout] enefunc : structure of potential energy function
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine dealloc_enefunc_all(enefunc)

    ! formal arguments
    type(s_enefunc),         intent(inout) :: enefunc


    call dealloc_enefunc(enefunc, EneFuncBond)
    call dealloc_enefunc(enefunc, EneFuncAngl)
    call dealloc_enefunc(enefunc, EneFuncUrey)
    call dealloc_enefunc(enefunc, EneFuncDihe)
    call dealloc_enefunc(enefunc, EneFuncImpr)
    call dealloc_enefunc(enefunc, EneFuncCmap)
    call dealloc_enefunc(enefunc, EneFuncNbon)
    call dealloc_enefunc(enefunc, EneFuncCntc)
    call dealloc_enefunc(enefunc, EneFuncRefg)
    call dealloc_enefunc(enefunc, EneFuncReff)
    call dealloc_enefunc(enefunc, EneFuncRefw)
    call dealloc_enefunc(enefunc, EneFuncRefc)
    call dealloc_enefunc(enefunc, EneFuncTable)
    call dealloc_enefunc(enefunc, EneFuncTableWater)
    call dealloc_enefunc(enefunc, EneFuncTableSolute)
    call dealloc_enefunc(enefunc, EneFuncCmapType)
    call dealloc_enefunc(enefunc, EneFuncPme)

    return

  end subroutine dealloc_enefunc_all

end module at_enefunc_str_mod
