!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   sp_energy_str_mod
!> @brief   structure of energy
!! @authors Jaewoon Jung (JJ), Yuji Sugita (YS), Takaharu Mori (TM)
!  
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

module sp_energy_str_mod

  use messages_mod
  use mpi_parallel_mod
  use constants_mod

  implicit none
  private

  ! structures
  type, public :: s_energy
    real(wp)         :: total
    ! charmm
    real(wp)         :: bond
    real(wp)         :: angle
    real(wp)         :: urey_bradley
    real(wp)         :: dihedral
    real(wp)         :: improper
    real(wp)         :: cmap
    real(wp)         :: electrostatic
    real(wp)         :: van_der_waals
    ! restraint
    real(wp)         :: restraint_position
    real(wp)         :: restraint_distance
  end type s_energy

  ! parameters
  integer,      public, parameter :: ElectrostaticCutoff = 1
  integer,      public, parameter :: ElectrostaticPME    = 2

  character(*), public, parameter :: ElectrostaticTypes(2) = (/'CUTOFF', &
                                                               'PME   '/)

  ! subroutines
  public  :: init_energy

contains
  
  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    init_energy
  !> @brief        initialize potential energy
  !! @authors      YS, TM
  !! @param[out]   energy : energy information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine init_energy(energy)

    ! formal arguments
    type(s_energy),          intent(inout) :: energy


    energy%total              = 0.0_wp
    energy%bond               = 0.0_wp
    energy%angle              = 0.0_wp
    energy%urey_bradley       = 0.0_wp
    energy%dihedral           = 0.0_wp
    energy%improper           = 0.0_wp
    energy%cmap               = 0.0_wp
    energy%electrostatic      = 0.0_wp
    energy%van_der_waals      = 0.0_wp
    energy%restraint_position = 0.0_wp
    energy%restraint_distance = 0.0_wp

    return

  end subroutine init_energy

end module sp_energy_str_mod
